<?php

namespace App\Http\Controllers;

use Google\Cloud\SecretManager\V1\SecretManagerServiceClient;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Facades\Log;

class SetupController extends Controller
{
    /**
     * Handle the incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function __invoke(Request $request): \Illuminate\Http\Response
    {
        // Authorize this request
        $this->authorizeRequest($request);

        try {

            Log::debug('Starting: Run database migration');

            // run the migration
            Artisan::call('migrate', [
                '--force' => true,
            ]);

            Log::debug('Finished: Run database migration');

        } catch (\Exception $e)
        {
            // log the error
            Log::error($e);

            return response('not ok', 500);
        }

        return response('ok', 200);
    }

    private function authorizeRequest(Request $request)
    {
        // Get secret key
        $setupSecretKey = config('services.gcp.sm_setup_secret_key');

        // Get secret value from request
        abort_if($request->missing($setupSecretKey), '403', 'Authentication Token is missing');

        // Get project Id
        $projectId = config('services.gcp.project_id');

        // Create the Secret Manager client.
        $client = new SecretManagerServiceClient();

        // Build the resource name of the secret version.
        $name = $client->secretVersionName($projectId, $setupSecretKey, 'latest');

        // Access the secret version.
        $response = $client->accessSecretVersion($name);

        abort_if(!$response, 403, 'Authentication Token is missing');

        // Get secret value
        $setupSecretValue = $response->getPayload()->getData();

        // Get secret value from current request
        $incomingSecretValue = $request->input($setupSecretKey);

        // only App Engine request pass through
        abort_unless(
            $setupSecretValue === $incomingSecretValue,
            403,
            'You are not authorized to perform this action'
        );
    }
}
